@@ -254,6 +254,13 @@ class ChunkedArray(awkward.array.base.AwkwardArray):
 
         return tpe
 
+    def _getnbytes(self, seen):
+        if id(self) in seen:
+            return 0
+        else:
+            seen.add(id(self))
+            return sum(x.nbytes if isinstance(x, self.numpy.ndarray) else x._getnbytes(seen) for x in self._chunks)
+
     def __len__(self):
         self.knowcounts()
         return self.offsets[-1]
